#!/usr/bin/env python

import micropython
from micropython.rsvp import raw
import rsvp
import sys
import os

code = None

def show(importer, with_builtins=0, optimisations=None):
    make(importer, with_builtins, optimisations)
    show_code(code)

def make(importer, with_builtins=0, optimisations=None):
    optimisations = optimisations or requested_optimisations
    global code
    code = importer.get_image(with_builtins, optimisations)

def show_code(code):
    for i, x in enumerate(code):
        print i, x

def show_table_usage(raw_table, slice_size=100):
    for x in xrange(0, len(raw_table), slice_size):
        table_slice = raw_table[x:x+slice_size]
        print "%6d" % (len(table_slice) - table_slice.count(None)), \
            "".join(entry and "#" or "_" for entry in table_slice)

def machine(importer):
    print "Making the image..."
    make(importer)
    rc = raw(importer.get_image())
    print "Getting raw tables..."
    objtable = importer.get_object_table().as_raw()
    paramtable = importer.get_parameter_table().as_raw()
    print "Initialising the machine..."
    rm = rsvp.RSVPMachine(rc, objtable, paramtable)
    rm.pc = importer.code_location
    return rm

def attrs(obj):
    for name, attr in obj.items():
        print name, attr

if __name__ == "__main__":
    args = sys.argv[2:]
    path = sys.path[:]

    if len(sys.argv) > 1:
        program = os.path.abspath(sys.argv[1])
        path.append(os.path.split(program)[0])
    else:
        program = None

    i = micropython.Importer(path, "-v" in args)

    if "-omax" in args:
        requested_optimisations = i.supported_optimisations
    else:
        requested_optimisations = []
        for arg in args:
            if arg.startswith("-o"):
                for arg_part in arg[2:].split(","):
                    requested_optimisations.append(arg_part)

    try:
        builtins = i.load_from_file("lib/builtins.py", "__builtins__")
        if program is None:
            print "Loading module micropython ..."
            m = i.load("micropython")
        else:
            print "Loading program", program, "..."
            m = i.load_from_file(program)
    except micropython.ProcessingError, exc:
        print repr(exc)
        if "-tb" in args:
            raise
    else:
        i.vacuum()
        ot = i.get_object_table()
        pt = i.get_parameter_table()

# vim: tabstop=4 expandtab shiftwidth=4
